import { NsTypography, Show } from '@nuskin/foundation-ui-components';
import equals from 'ramda/src/equals';
import { ReactElement, memo } from 'react';

import { ShowcaseProps } from '../../types/types';
import { ShowcaseImageContainer } from '../ShowcaseCommon.styled';
import {
    ShowcaseAccentImageContainer,
    ShowcaseDefaultContentContainer,
    ShowcaseDefaultTextContainer,
    ShowcasePreHeader
} from './Showcase.styled';
import { BynderImage } from '../../../bynder-image/BynderImage';
import { getBasicDATWidthHeight } from '../../../utils/bynder-util';
import { NsCTALink } from '../../../cta';
import { useMainContrast, useToggleFontColor } from '../../../hooks';

/**
 * ns showcase global field https://app.contentstack.com/#!/stack/blt7d4c4f4a1bf5a819/global-field/ns_showcase/global-field-builder?branch=main
 * Showcase renders an image and text panel where its orientation can be controlled by image alignment.
 * It also features an accent image. The text panel slightly overlaps on the image.
 * Key properties: pre-header, header, description, cta, image panel, accent image.
 */
const suffix = getBasicDATWidthHeight(600, 540, 'scale');
const accentSuffix = getBasicDATWidthHeight(160, 160, 'scale');
const Showcase = memo((props: ShowcaseProps): ReactElement => {
    const {
        preHeader,
        preHeaderBackgroundColor,
        header,
        description,
        fontColor,
        backgroundColor,
        image,
        alignment,
        accentImage,
        callToAction,
        liveTags,
        dataTestId = 'showcase-default'
    } = props;

    const { main: preHeaderBg, contrast: preHeaderFont } = useMainContrast(preHeaderBackgroundColor);

    const resolvedFontColor = useToggleFontColor(fontColor);
    const accentImageContent = (
        <ShowcaseAccentImageContainer className="accent-image-container">
            <BynderImage testId="accent-image" className="accent-image" imageText={accentImage} suffix={accentSuffix} />
        </ShowcaseAccentImageContainer>
    );
    const textContent = (
        <ShowcaseDefaultTextContainer backgroundColor={backgroundColor} className="showcase-default-text-container">
            <Show when={!!preHeader}>
                <ShowcasePreHeader
                    variant="body-m"
                    backgroundColor={preHeaderBg}
                    colorOverride={preHeaderFont}
                    className="showcase-pre-header pre-header"
                    additionalProps={{
                        'data-testid': 'pre-header',
                        ...liveTags?.preHeader
                    }}
                    noSpacing
                >
                    {preHeader}
                </ShowcasePreHeader>
            </Show>
            <Show when={!!header}>
                <NsTypography
                    variant="h4"
                    colorOverride={resolvedFontColor}
                    className="showcase-header"
                    additionalProps={{
                        'data-testid': 'header',
                        ...liveTags?.header
                    }}
                    noSpacing
                >
                    {header}
                </NsTypography>
            </Show>
            <Show when={!!description}>
                <NsTypography
                    variant="body-l"
                    colorOverride={resolvedFontColor}
                    className="description"
                    additionalProps={{
                        'data-testid': 'description',
                        ...liveTags?.description
                    }}
                    noSpacing
                >
                    {description}
                </NsTypography>
            </Show>
            <Show when={!!callToAction?.url && !!callToAction?.text}>
                <NsCTALink
                    ctaProps={{ ...callToAction, color: fontColor ? 'secondary' : 'primary' }}
                    liveTag={liveTags?.callToAction}
                />
            </Show>
            {accentImageContent}
        </ShowcaseDefaultTextContainer>
    );

    const className = [`image-align-${alignment === 'left' ? 'left' : 'right'}`, accentImage && 'has-accent']
        .filter(Boolean)
        .join(' ');

    return (
        <section className="showcase-default" data-testid={dataTestId}>
            <ShowcaseDefaultContentContainer
                backgroundColor="transparent"
                className={`${className} showcase-default-content-container`}
            >
                {textContent}
                <ShowcaseImageContainer className="showcase-image-container">
                    <BynderImage
                        imageText={image ?? ''}
                        suffix={suffix}
                        liveTag={liveTags?.image ?? { 'data-cslp': '' }}
                    />
                </ShowcaseImageContainer>
                {accentImageContent}
            </ShowcaseDefaultContentContainer>
        </section>
    );
}, equals);

Showcase.displayName = 'Showcase';

export default Showcase;
