import { GetServerSidePropsContext } from 'next';

import referenceMap from '../nuskin-content-stack/reference-map';
import {
    fetchStoreCategory,
    getContentTypeData,
    getDomainAndLocale,
    getPageContent,
    getRegionAndLanguageFromQuery,
} from '.';

export const ns_default_header: string = 'ns_default_header';
export const ns_ffg_footer: string = 'ns_ffg_footer';
export const ns_default_utilitybar: string = 'ns_default_utilitybar';

const getReference = (property: string, header: any, footer: any, utilityBar: any) => {
    switch (property) {
        case 'footer':
            if (footer?.value) return { ...footer?.value, _content_type_uid: ns_ffg_footer };
            break;
        case 'header':
            if (header?.value)
                return {
                    _content_type_uid: ns_default_header,
                    ...header?.value,
                };
            break;
        case 'utility_bar':
            if (utilityBar?.value) return { ...utilityBar?.value, _content_type_uid: ns_default_utilitybar };
            break;
        default:
            return null;
    }
};

export const processReferences = (pageRes: any, header: any, footer: any, utilityBar: any) => {
    const propertiesToProcess = ['footer', 'header', 'utility_bar'];
    propertiesToProcess.forEach((property) => {
        if (pageRes?.value?.[property]) {
            const reference = getReference(property, header, footer, utilityBar);
            pageRes.value[property] = {
                ...(pageRes.value[property] || {}),
                ...{ [`${property}_reference`]: [reference] },
            };
        }
    });
    return pageRes;
};

export const getNuskinPagesServerSideProps = async (
    context: GetServerSidePropsContext,
    baseUrl: string,
    pageContentUUID: string,
    pageSlug?: string
) => {
    const { query } = context;
    const querySlug = (query?.slug || ['/']) as string[];
    const slug = `${baseUrl}/${pageSlug || querySlug?.join('/')}`;

    const { languageCode, domain, locale } = getDomainAndLocale(context);
    const fetchPageContent = getPageContent(pageContentUUID, languageCode, slug, domain);
    const regionLanguage = getRegionAndLanguageFromQuery(query);
    const headerData = getContentTypeData(ns_default_header, referenceMap.ns_default_header);
    const footerData = getContentTypeData(ns_ffg_footer);
    const utilityBarData = getContentTypeData(ns_default_utilitybar);

    const fetchCategory = fetchStoreCategory(locale);
    // TODO: Replace any with proper type
    let [pageRes, headerRes, header, footer, utilityBar]: any = await Promise.allSettled([
        fetchPageContent,
        fetchCategory,
        headerData,
        footerData,
        utilityBarData,
    ]);

    pageRes = processReferences(pageRes, header, footer, utilityBar);

    if (!pageRes?.value) {
        return {
            notFound: true,
        };
    }

    return {
        props: {
            ...pageRes?.value,
            headerRes: headerRes?.value ?? null,
            theme: pageRes?.value?.site_properties?.[0]?.theme || '',
            locale,
            regionLanguage,
            headerData: header,
            footerData: footer,
            utilityBarData: utilityBar,
            fetchProps: {
                pageName: pageContentUUID,
                languageCode,
                slug,
                domain,
            },
        },
    };
};
